"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  createHooks: () => createHooks
});
module.exports = __toCommonJS(src_exports);

// src/hooks.ts
var import_vue = require("vue");
var import_vue_query = require("@tanstack/vue-query");
var import_internal = require("@wundergraph/sdk/internal");
var userQueryKey = "wg_user";
var createHooks = (client) => {
  const queryFetcher = async (query) => {
    const result = await client.query(query);
    if (result.error) {
      throw result.error;
    }
    return result.data;
  };
  const mutationFetcher = async (mutation) => {
    const result = await client.mutate(mutation);
    if (result.error) {
      throw result.error;
    }
    return result.data;
  };
  const queryKey = ({ operationName, input }) => {
    return [operationName, input];
  };
  const useQuery = (options) => {
    const { operationName, liveQuery, input, enabled, refetchOnWindowFocus, ...queryOptions } = options;
    const queryHash = (0, import_internal.serialize)([operationName, input]);
    const result = (0, import_vue_query.useQuery)({
      queryKey: queryKey({ operationName, input }),
      queryFn: ({ signal }) => queryFetcher({ operationName, input, abortSignal: signal }),
      ...queryOptions,
      enabled: liveQuery ? false : enabled,
      refetchOnWindowFocus: liveQuery ? false : refetchOnWindowFocus
    });
    const { isSubscribed } = useSubscribeTo(
      (0, import_vue.reactive)({
        queryHash,
        operationName,
        input,
        liveQuery,
        enabled: options.enabled !== false && liveQuery,
        onSuccess: (0, import_vue.unref)(options.onSuccess),
        onError: (0, import_vue.unref)(options.onError)
      })
    );
    if (liveQuery) {
      return {
        ...result,
        isSubscribed
      };
    }
    return result;
  };
  const useMutation = (options) => {
    const { operationName, variables, ...mutationOptions } = options;
    const result = (0, import_vue_query.useMutation)([operationName], (input) => mutationFetcher({ operationName, input }), {
      ...mutationOptions,
      variables
      //Bug with the vue-query types
    });
    return result;
  };
  const useAuth = () => {
    const queryClient = (0, import_vue_query.useQueryClient)();
    return {
      login: (authProviderID, redirectURI) => client.login(authProviderID, redirectURI),
      logout: async (options) => {
        const result = await client.logout(options);
        queryClient.setQueryData([userQueryKey], null);
        return result;
      }
    };
  };
  const useUser = (options) => {
    const { revalidate, ...queryOptions } = options || {};
    return (0, import_vue_query.useQuery)(
      [userQueryKey],
      ({ signal }) => client.fetchUser({
        revalidate,
        abortSignal: signal
      }),
      queryOptions
    );
  };
  const useFileUpload = (options) => {
    const { mutate, mutateAsync, ...mutation } = (0, import_vue_query.useMutation)(
      ["uploadFiles"],
      async (input) => {
        const resp = await client.uploadFiles(input);
        return resp.fileKeys;
      },
      options
    );
    return {
      upload: mutate,
      uploadAsync: mutateAsync,
      ...mutation
    };
  };
  const subscribeTo = (options) => {
    const abort = new AbortController();
    const { onSuccess, onError, onResult, onAbort, ...subscription } = options;
    subscription.abortSignal = abort.signal;
    client.subscribe(subscription, onResult).catch(onError);
    return () => {
      onAbort == null ? void 0 : onAbort();
      abort.abort();
    };
  };
  const useSubscribeTo = (props) => {
    const client2 = (0, import_vue_query.useQueryClient)();
    const { queryHash, operationName, input, enabled, liveQuery, subscribeOnce, resetOnMount, onSuccess, onError } = (0, import_vue.unref)(props);
    const startedAt = (0, import_vue.ref)(null);
    const isLoading = (0, import_vue.ref)(false);
    const isSubscribed = (0, import_vue.ref)(false);
    let unsubscribe = () => void 0;
    (0, import_vue.onMounted)(() => {
      if (!startedAt.value && resetOnMount) {
        client2.removeQueries([operationName, input]);
      }
      if (enabled) {
        isLoading.value = true;
        isSubscribed.value = false;
        unsubscribe = subscribeTo({
          operationName,
          input,
          liveQuery: (0, import_vue.unref)(liveQuery),
          subscribeOnce,
          onError(error) {
            isLoading.value = false;
            isSubscribed.value = false;
            onError == null ? void 0 : onError(error);
            startedAt.value = null;
          },
          onResult(result) {
            if (!startedAt.value) {
              isLoading.value = false;
              isSubscribed.value = true;
              onSuccess == null ? void 0 : onSuccess(result);
              startedAt.value = (/* @__PURE__ */ new Date()).getTime();
            }
            client2.setQueryData([operationName, input], () => {
              if (result.error)
                throw result.error;
              return result.data;
            });
          },
          onAbort() {
            isLoading.value = false;
            isSubscribed.value = false;
            startedAt.value = null;
          }
        });
      }
    });
    (0, import_vue.onUnmounted)(() => {
      unsubscribe();
    });
    return { isLoading, isSubscribed };
  };
  const useSubscription = (options) => {
    const { enabled = true, operationName, input, subscribeOnce, onSuccess, onError } = options;
    const queryHash = (0, import_internal.serialize)([operationName, input]);
    const subscription = (0, import_vue_query.useQuery)({
      queryKey: [operationName, input],
      enabled: false
      // we update the cache async
    });
    const { isSubscribed } = useSubscribeTo(
      (0, import_vue.reactive)({
        queryHash,
        operationName,
        input,
        subscribeOnce,
        enabled,
        onSuccess,
        onError
      })
    );
    return {
      ...subscription,
      isSubscribed
    };
  };
  return {
    useAuth,
    useFileUpload,
    useUser,
    useMutation,
    useQuery,
    useSubscription,
    queryKey
  };
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  createHooks
});
